@@ -21,11 +21,7 @@ from .utils import operator as op
 from .utils.logging import get_tb_logger
 
 
-__all__ = [
-    "_BaseGradientBoosting",
-    "GradientBoostingClassifier",
-    "GradientBoostingRegressor",
-]
+__all__ = ["GradientBoostingClassifier", "GradientBoostingRegressor"]
 
 
 __model_doc = """
@@ -190,10 +186,11 @@ class _BaseGradientBoosting(BaseModule):
     def _handle_early_stopping(self, test_loader, est_idx):
         """Decide whether to trigger the internal counter on early stopping."""
 
-    def _staged_forward(self, x, est_idx):
+    def _staged_forward(self, est_idx, *x):
         """
         Return the accumulated outputs from the first `est_idx+1` base
-        estimators."""
+        estimators.
+        """
         if est_idx >= self.n_estimators:
             msg = (
                 "est_idx = {} should be an integer smaller than the"
@@ -203,29 +200,12 @@ class _BaseGradientBoosting(BaseModule):
             raise ValueError(msg.format(est_idx, self.n_estimators))
 
         outputs = [
-            estimator(x) for estimator in self.estimators_[: est_idx + 1]
+            estimator(*x) for estimator in self.estimators_[: est_idx + 1]
         ]
         out = op.sum_with_multiplicative(outputs, self.shrinkage_rate)
 
         return out
 
-    @torchensemble_model_doc(
-        """Set the attributes on optimizer for Gradient Boosting.""",
-        "set_optimizer",
-    )
-    def set_optimizer(self, optimizer_name, **kwargs):
-        self.optimizer_name = optimizer_name
-        self.optimizer_args = kwargs
-
-    @torchensemble_model_doc(
-        """Set the attributes on scheduler for Gradient Boosting.""",
-        "set_scheduler",
-    )
-    def set_scheduler(self, scheduler_name, **kwargs):
-        self.scheduler_name = scheduler_name
-        self.scheduler_args = kwargs
-        self.use_scheduler_ = True
-
     def fit(
         self,
         train_loader,
@@ -242,9 +222,7 @@ class _BaseGradientBoosting(BaseModule):
         for _ in range(self.n_estimators):
             self.estimators_.append(self._make_estimator())
         self._validate_parameters(epochs, log_interval, early_stopping_rounds)
-        self.n_outputs = self._decide_n_outputs(
-            train_loader, self.is_classification
-        )
+        self.n_outputs = self._decide_n_outputs(train_loader)
 
         # Utils
         criterion = (
@@ -271,14 +249,14 @@ class _BaseGradientBoosting(BaseModule):
             estimator.train()
             total_iters = 0
             for epoch in range(epochs):
-                for batch_idx, (data, target) in enumerate(train_loader):
+                for batch_idx, elem in enumerate(train_loader):
 
-                    data, target = data.to(self.device), target.to(self.device)
+                    data, target = io.split_data_target(elem, self.device)
 
                     # Compute the learning target of the current estimator
-                    residual = self._pseudo_residual(data, target, est_idx)
+                    residual = self._pseudo_residual(est_idx, target, *data)
 
-                    output = estimator(data)
+                    output = estimator(*data)
                     loss = criterion(output, residual)
 
                     learner_optimizer.zero_grad()
@@ -342,19 +320,15 @@ class _BaseGradientBoosting(BaseModule):
     """Implementation on the GradientBoostingClassifier.""", "model"
 )
 class GradientBoostingClassifier(_BaseGradientBoosting, BaseClassifier):
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        self.is_classification = True
-
-    def _pseudo_residual(self, X, y, est_idx):
+    def _pseudo_residual(self, est_idx, y, *x):
         """Compute pseudo residuals in classification."""
-        output = torch.zeros(X.size(0), self.n_outputs).to(self.device)
+        output = torch.zeros(y.size(0), self.n_outputs).to(self.device)
 
         # Before fitting the first estimator, we simply assume that GBM
         # outputs 0 for any input (i.e., a null output).
         if est_idx > 0:
             results = [
-                estimator(X) for estimator in self.estimators_[:est_idx]
+                estimator(*x) for estimator in self.estimators_[:est_idx]
             ]
             output += op.sum_with_multiplicative(results, self.shrinkage_rate)
         pseudo_residual = op.pseudo_residual_classification(
@@ -370,9 +344,9 @@ class GradientBoostingClassifier(_BaseGradientBoosting, BaseClassifier):
         total = 0
         flag = False
         with torch.no_grad():
-            for _, (data, target) in enumerate(test_loader):
-                data, target = data.to(self.device), target.to(self.device)
-                output = F.softmax(self._staged_forward(data, est_idx), dim=1)
+            for _, elem in enumerate(test_loader):
+                data, target = io.split_data_target(elem, self.device)
+                output = F.softmax(self._staged_forward(est_idx, *data), dim=1)
                 _, predicted = torch.max(output.data, 1)
                 correct += (predicted == target).sum().item()
                 total += target.size(0)
@@ -400,14 +374,14 @@ class GradientBoostingClassifier(_BaseGradientBoosting, BaseClassifier):
         "set_optimizer",
     )
     def set_optimizer(self, optimizer_name, **kwargs):
-        super().set_optimizer(optimizer_name=optimizer_name, **kwargs)
+        super().set_optimizer(optimizer_name, **kwargs)
 
     @torchensemble_model_doc(
         """Set the attributes on scheduler for GradientBoostingClassifier.""",
         "set_scheduler",
     )
     def set_scheduler(self, scheduler_name, **kwargs):
-        super().set_scheduler(scheduler_name=scheduler_name, **kwargs)
+        super().set_scheduler(scheduler_name, **kwargs)
 
     @_gradient_boosting_model_doc(
         """Implementation on the training stage of GradientBoostingClassifier.""",  # noqa: E501
@@ -424,6 +398,7 @@ class GradientBoostingClassifier(_BaseGradientBoosting, BaseClassifier):
         save_model=True,
         save_dir=None,
     ):
+        self._criterion = nn.CrossEntropyLoss()
         super().fit(
             train_loader=train_loader,
             epochs=epochs,
@@ -439,8 +414,8 @@ class GradientBoostingClassifier(_BaseGradientBoosting, BaseClassifier):
         """Implementation on the data forwarding in GradientBoostingClassifier.""",  # noqa: E501
         "classifier_forward",
     )
-    def forward(self, x):
-        output = [estimator(x) for estimator in self.estimators_]
+    def forward(self, *x):
+        output = [estimator(*x) for estimator in self.estimators_]
         output = op.sum_with_multiplicative(output, self.shrinkage_rate)
         proba = F.softmax(output, dim=1)
 
@@ -451,25 +426,21 @@ class GradientBoostingClassifier(_BaseGradientBoosting, BaseClassifier):
         return super().evaluate(test_loader, return_loss)
 
     @torchensemble_model_doc(item="predict")
-    def predict(self, X, return_numpy=True):
-        return super().predict(X, return_numpy)
+    def predict(self, *x):
+        return super().predict(*x)
 
 
 @_gradient_boosting_model_doc(
     """Implementation on the GradientBoostingRegressor.""", "model"
 )
 class GradientBoostingRegressor(_BaseGradientBoosting, BaseRegressor):
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        self.is_classification = False
-
-    def _pseudo_residual(self, X, y, est_idx):
+    def _pseudo_residual(self, est_idx, y, *x):
         """Compute pseudo residuals in regression."""
         output = torch.zeros_like(y).to(self.device)
 
         if est_idx > 0:
             results = [
-                estimator(X) for estimator in self.estimators_[:est_idx]
+                estimator(*x) for estimator in self.estimators_[:est_idx]
             ]
             output = op.sum_with_multiplicative(results, self.shrinkage_rate)
         pseudo_residual = op.pseudo_residual_regression(y, output)
@@ -483,9 +454,9 @@ class GradientBoostingRegressor(_BaseGradientBoosting, BaseRegressor):
         flag = False
         criterion = nn.MSELoss()
         with torch.no_grad():
-            for _, (data, target) in enumerate(test_loader):
-                data, target = data.to(self.device), target.to(self.device)
-                output = self._staged_forward(data, est_idx)
+            for _, elem in enumerate(test_loader):
+                data, target = io.split_data_target(elem, self.device)
+                output = self._staged_forward(est_idx, *data)
                 mse += criterion(output, target)
         mse /= len(test_loader)
 
@@ -512,14 +483,14 @@ class GradientBoostingRegressor(_BaseGradientBoosting, BaseRegressor):
         "set_optimizer",
     )
     def set_optimizer(self, optimizer_name, **kwargs):
-        super().set_optimizer(optimizer_name=optimizer_name, **kwargs)
+        super().set_optimizer(optimizer_name, **kwargs)
 
     @torchensemble_model_doc(
         """Set the attributes on scheduler for GradientBoostingRegressor.""",
         "set_scheduler",
     )
     def set_scheduler(self, scheduler_name, **kwargs):
-        super().set_scheduler(scheduler_name=scheduler_name, **kwargs)
+        super().set_scheduler(scheduler_name, **kwargs)
 
     @_gradient_boosting_model_doc(
         """Implementation on the training stage of GradientBoostingRegressor.""",  # noqa: E501
@@ -536,6 +507,7 @@ class GradientBoostingRegressor(_BaseGradientBoosting, BaseRegressor):
         save_model=True,
         save_dir=None,
     ):
+        self._criterion = nn.MSELoss()
         super().fit(
             train_loader=train_loader,
             epochs=epochs,
@@ -551,8 +523,8 @@ class GradientBoostingRegressor(_BaseGradientBoosting, BaseRegressor):
         """Implementation on the data forwarding in GradientBoostingRegressor.""",  # noqa: E501
         "regressor_forward",
     )
-    def forward(self, x):
-        outputs = [estimator(x) for estimator in self.estimators_]
+    def forward(self, *x):
+        outputs = [estimator(*x) for estimator in self.estimators_]
         pred = op.sum_with_multiplicative(outputs, self.shrinkage_rate)
 
         return pred
@@ -562,5 +534,5 @@ class GradientBoostingRegressor(_BaseGradientBoosting, BaseRegressor):
         return super().evaluate(test_loader)
 
     @torchensemble_model_doc(item="predict")
-    def predict(self, X, return_numpy=True):
-        return super().predict(X, return_numpy)
+    def predict(self, *x):
+        return super().predict(*x)
